/*************************************************************************
*
*			Date 对象的扩展
*
**************************************************************************/
Date.prototype.Format = function (format) {
    /// <summary>格式化时间为字符串</summary>
    /// <param name="format" type="string">必需。例如："yyyy-MM-dd hh:mm:ss",其中M表示月，d表示日，h表示小时，m表示分钟，s表示秒，q表示季度，S表示毫秒</param>
    /// <returns type="string">返回对应格式时间</returns>
    var o = {
        "M+": this.getMonth() + 1,  //month
        "d+": this.getDate(),     //day
        "h+": this.getHours(),    //hour
        "m+": this.getMinutes(),  //minute
        "s+": this.getSeconds(), //second
        "q+": Math.floor((this.getMonth() + 3) / 3),  //quarter
        "S": this.getMilliseconds() //millisecond
    }

    if (/(y+)/.test(format)) {
        format = format.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    }

    for (var k in o) {
        if (new RegExp("(" + k + ")").test(format)) {
            format = format.replace(RegExp.$1, RegExp.$1.length == 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length));
        }
    }
    return format;
}

Date.prototype.DateAdd = function (interval, number) {
    /// <summary>计算时间偏移,例如:var tc = now.DateAdd("y",1);</summary>
    /// <param name="interval" type="string">必需。y表示年，m表示年，d表示日，w表示周，h表示小时，n表示分钟，s表示秒，l表示毫秒</param>
    /// <param name="number" type="int">必需。可以为负值</param>
    /// <returns type="datetime">返回时间对象</returns>
    number = parseInt(number);
    var date = new Date(this.getTime());
    //
    switch (interval) {
        case "y": date.setFullYear(this.getFullYear() + number); break;
        case "m": date.setMonth(this.getMonth() + number); break;
        case "d": date.setDate(this.getDate() + number); break;
        case "w": date.setDate(this.getDate() + 7 * number); break;
        case "h": date.setHours(this.getHours() + number); break;
        case "n": date.setMinutes(this.getMinutes() + number); break;
        case "s": date.setSeconds(this.getSeconds() + number); break;
        case "l": date.setMilliseconds(this.getMilliseconds() + number); break;
    }
    return date;
}


// 计算当前日期在本年度的周数  
Date.prototype.getWeekOfYear = function(weekStart) { // weekStart：每周开始于周几：周日：0，周一：1，周二：2 ...，默认为周日  
    weekStart = (weekStart || 0) - 0;  
    if(isNaN(weekStart) || weekStart > 6)  
        weekStart = 0;    
  
    var year = this.getFullYear();  
    var firstDay = new Date(year, 0, 1);  
    var firstWeekDays = 7 - firstDay.getDay() + weekStart;  
    var dayOfYear = (((new Date(year, this.getMonth(), this.getDate())) - firstDay) / (24 * 3600 * 1000)) + 1;  
    return Math.ceil((dayOfYear - firstWeekDays) / 7) + 1;  
}  
  
// 计算当前日期在本月份的周数  
Date.prototype.getWeekOfMonth = function(weekStart) {  
    weekStart = (weekStart || 0) - 0;  
    if(isNaN(weekStart) || weekStart > 6)  
        weekStart = 0;  
  
    var dayOfWeek = this.getDay();  
    var day = this.getDate();  
    return Math.ceil((day - dayOfWeek - 1) / 7) + ((dayOfWeek >= weekStart) ? 1 : 0);  
}
//计算农历节假返回的中文
Date.prototype.getLunar = function(type) { 
	var lunarInfo=new Array(
		0x04bd8,0x04ae0,0x0a570,0x054d5,0x0d260,0x0d950,0x16554,0x056a0,0x09ad0,0x055d2,
		0x04ae0,0x0a5b6,0x0a4d0,0x0d250,0x1d255,0x0b540,0x0d6a0,0x0ada2,0x095b0,0x14977,
		0x04970,0x0a4b0,0x0b4b5,0x06a50,0x06d40,0x1ab54,0x02b60,0x09570,0x052f2,0x04970,
		0x06566,0x0d4a0,0x0ea50,0x06e95,0x05ad0,0x02b60,0x186e3,0x092e0,0x1c8d7,0x0c950,
		0x0d4a0,0x1d8a6,0x0b550,0x056a0,0x1a5b4,0x025d0,0x092d0,0x0d2b2,0x0a950,0x0b557,
		0x06ca0,0x0b550,0x15355,0x04da0,0x0a5d0,0x14573,0x052d0,0x0a9a8,0x0e950,0x06aa0,
		0x0aea6,0x0ab50,0x04b60,0x0aae4,0x0a570,0x05260,0x0f263,0x0d950,0x05b57,0x056a0,
		0x096d0,0x04dd5,0x04ad0,0x0a4d0,0x0d4d4,0x0d250,0x0d558,0x0b540,0x0b5a0,0x195a6,
		0x095b0,0x049b0,0x0a974,0x0a4b0,0x0b27a,0x06a50,0x06d40,0x0af46,0x0ab60,0x09570,
		0x04af5,0x04970,0x064b0,0x074a3,0x0ea50,0x06b58,0x055c0,0x0ab60,0x096d5,0x092e0,
		0x0c960,0x0d954,0x0d4a0,0x0da50,0x07552,0x056a0,0x0abb7,0x025d0,0x092d0,0x0cab5,
		0x0a950,0x0b4a0,0x0baa4,0x0ad50,0x055d9,0x04ba0,0x0a5b0,0x15176,0x052b0,0x0a930,
		0x07954,0x06aa0,0x0ad50,0x05b52,0x04b60,0x0a6e6,0x0a4e0,0x0d260,0x0ea65,0x0d530,
		0x05aa0,0x076a3,0x096d0,0x04bd7,0x04ad0,0x0a4d0,0x1d0b6,0x0d250,0x0d520,0x0dd45,
		0x0b5a0,0x056d0,0x055b2,0x049b0,0x0a577,0x0a4b0,0x0aa50,0x1b255,0x06d20,0x0ada0);
	var Animals=new Array("鼠","牛","虎","兔","龙","蛇","马","羊","猴","鸡","狗","猪");
	var Gan=new Array("甲","乙","丙","丁","戊","己","庚","辛","壬","癸");
	var Zhi=new Array("子","丑","寅","卯","辰","巳","午","未","申","酉","戌","亥");
	var now = new Date(this.getTime());
	var SY = now.getFullYear(); 
	var SM = now.getMonth();
	var SD = now.getDate();
 
	//==== 传入 offset 传回干支, 0=甲子
	function cyclical(num) { return(Gan[num%10]+Zhi[num%12])}

	//==== 传回农历 y年的总天数
	function lYearDays(y) {
	   var i, sum = 348
	   for(i=0x8000; i>0x8; i>>=1) sum += (lunarInfo[y-1900] & i)? 1: 0
	   return(sum+leapDays(y))
	}

	//==== 传回农历 y年闰月的天数
	function leapDays(y) {
	   if(leapMonth(y))  return((lunarInfo[y-1900] & 0x10000)? 30: 29)
	   else return(0)
	}

	//==== 传回农历 y年闰哪个月 1-12 , 没闰传回 0
	function leapMonth(y) { return(lunarInfo[y-1900] & 0xf)}

	//====================================== 传回农历 y年m月的总天数
	function monthDays(y,m) { return( (lunarInfo[y-1900] & (0x10000>>m))? 30: 29 )}

	//==== 算出农历, 传入日期物件, 传回农历日期物件
	//     该物件属性有 .year .month .day .isLeap .yearCyl .dayCyl .monCyl
	function Lunar(objDate) {
	   var i, leap=0, temp=0
	   var baseDate = new Date(1900,0,31)
	   var offset   = (objDate - baseDate)/86400000

	   this.dayCyl = offset + 40
	   this.monCyl = 14

	   for(i=1900; i<2050 && offset>0; i++) {
		  temp = lYearDays(i)
		  offset -= temp
		  this.monCyl += 12
	   }
	   if(offset<0) {
		  offset += temp;
		  i--;
		  this.monCyl -= 12
	   }

	   this.year = i
	   this.yearCyl = i-1864

	   leap = leapMonth(i) //闰哪个月
	   this.isLeap = false

	   for(i=1; i<13 && offset>0; i++) {
		  //闰月
		  if(leap>0 && i==(leap+1) && this.isLeap==false)
			 { --i; this.isLeap = true; temp = leapDays(this.year); }
		  else
			 { temp = monthDays(this.year, i); }

		  //解除闰月
		  if(this.isLeap==true && i==(leap+1)) this.isLeap = false

		  offset -= temp
		  if(this.isLeap == false) this.monCyl ++
	   }

	   if(offset==0 && leap>0 && i==leap+1)
		  if(this.isLeap)
			 { this.isLeap = false; }
		  else
			 { this.isLeap = true; --i; --this.monCyl;}

	   if(offset<0){ offset += temp; --i; --this.monCyl; }

	   this.month = i
	   this.day = offset + 1
	}

	 function YYMMDD(){ 
		return(SY+'年'+(SM+1)+'月'+SD+'日'); 
	 }
	 function weekday(){ 
		var day = new Array("星期日","星期一","星期二","星期三","星期四","星期五","星期六");
		return(day[now.getDay()]); 
	 }
	//==== 中文日期
	function cDay(m,d){
	 var nStr1 = new Array('日','一','二','三','四','五','六','七','八','九','十');
	 var nStr2 = new Array('初','十','廿','卅','　');
	 var s;
	 if (m>10){s = '十'+nStr1[m-10]} else {s = nStr1[m]} s += '月'
	 switch (d) {
	  case 10:s += '初十'; break;
	  case 20:s += '二十'; break;
	  case 30:s += '三十'; break;
	  default:s += nStr2[Math.floor(d/10)]; s += nStr1[d%10];
	 }
	 return(s);
	}

	function lDay(d){
	 var nStr1 = new Array('日','一','二','三','四','五','六','七','八','九','十');
	 var nStr2 = new Array('初','十','廿','卅','　');
	 var s='';
	 switch (d) {
	  case 10:s += '初十'; break;
	  case 20:s += '二十'; break;
	  case 30:s += '三十'; break;
	  default:s += nStr2[Math.floor(d/10)]; s += nStr1[d%10];
	 }
	 return(s);
	}

	switch(type){
		case 0:
		{
			var sDObj = new Date(SY,SM,SD);
			var lDObj = new Lunar(sDObj);
			var tt = '【'+Animals[(SY-4)%12]+'】'+cyclical(lDObj.monCyl)+'月 '+cyclical(lDObj.dayCyl++)+'日' ;
			return tt;
		}
		break;
		case 1:
		{
			var sDObj = new Date(SY,SM,SD);
			var lDObj = new Lunar(sDObj);
			var tt = cyclical(SY-1900+36)+'年 '+cDay(lDObj.month,lDObj.day);
			return tt;
		}
		break;
		case 2:
		{
			var sTermInfo = new Array(0,21208,42467,63836,85337,107014,128867,150921,173149,195551,218072,240693,263343,285989,308563,331033,353350,375494,397447,419210,440795,462224,483532,504758);
			var solarTerm = new Array("小寒","大寒","立春","雨水","惊蛰","春分","清明","谷雨","立夏","小满","芒种","夏至","小暑","大暑","立秋","处暑","白露","秋分","寒露","霜降","立冬","小雪","大雪","冬至");
			var lFtv = new Array("0101*春节","0115 元宵节","0505 端午节","0707 七夕情人节","0715 中元节","0815 中秋节","0909 重阳节","1208 腊八节","1224 小年","0100*除夕");
			var sFtv =new Array("0101*元旦", "0214 情人节", "0308 妇女节", "0312 植树节", "0315 消费者权益日", "0401 愚人节", "0501 劳动节", "0504 青年节", "0512 护士节", "0601 儿童节", "0701 建党节 香港回归纪念", "0801 建军节", "0808 父亲节","0908 茂生日", "0909 毛泽东逝世纪念", "0910 教师节", "0928 孔子诞辰", "1001*国庆节", "1006 老人节", "1001 ListenWind生日", "1024 联合国日", "1112 孙中山诞辰", "1220 澳门回归纪念", "1225 圣诞节", "1226 毛泽东诞辰");

			var sDObj = new Date(SY,SM,SD);
			var lDObj = new Lunar(sDObj);
			var lDPOS = new Array(3);
			var festival='',solarTerms='',solarFestival='',lunarFestival='',tmp1,tmp2;
			//农历节日 
			for(i in lFtv)
				if(lFtv[i].match(/^(\d{2})(.{2})([\s\*])(.+)$/)) {
					tmp1=Number(RegExp.$1)-lDObj.month;
					tmp2=Number(RegExp.$2)-lDObj.day;
					if(tmp1==0 && tmp2==0) lunarFestival=RegExp.$4; 
			}
			//国历节日
			for(i in sFtv)
				if(sFtv[i].match(/^(\d{2})(\d{2})([\s\*])(.+)$/)){
					tmp1=Number(RegExp.$1)-(SM+1);
					tmp2=Number(RegExp.$2)-SD;
					if(tmp1==0 && tmp2==0) solarFestival = RegExp.$4 ;
			}
			//节气
			  tmp1 = new Date((31556925974.7*(SY-1900)+sTermInfo[SM*2+1]*60000)+Date.UTC(1900,0,6,2,5));
			  tmp2 = tmp1.getUTCDate();
			  if (tmp2==SD) solarTerms = solarTerm[SM*2+1]  
			  tmp1 = new Date((31556925974.7*(SY-1900)+sTermInfo[SM*2]*60000)+Date.UTC(1900,0,6,2,5))
			  tmp2= tmp1.getUTCDate();
			  if (tmp2==SD) solarTerms = solarTerm[SM*2];

			  if(lunarFestival=="春节")
				  return "祝新春愉快，阖家幸福！";

			  if(lunarFestival=="元宵节")
				  return "祝元宵节快乐！";

			  if(lunarFestival=="中秋节")
				  return "祝中秋幸福，团团圆圆！";

			  if(solarFestival=="元旦")
				  return "祝新年快乐，好运连年！";

			  if(solarFestival=="教师节")
				  return "祝所有教师节日快乐！";

			  if(solarFestival=="国庆节")
				  return "祝祖国繁荣昌盛，傲然世界东方！";

			  if(solarTerms == '' && solarFestival == '' && lunarFestival == '')
				festival = '【'+Animals[(SY-4)%12]+'】'+cDay(lDObj.month,lDObj.day);
			  else
				festival = '今天是'+solarTerms + ' ' + solarFestival + ' ' + lunarFestival;
					 
			  return festival;	
		}
		break;
		case 3:
		{
			var sTermInfo = new Array(0,21208,42467,63836,85337,107014,128867,150921,173149,195551,218072,240693,263343,285989,308563,331033,353350,375494,397447,419210,440795,462224,483532,504758);
			var solarTerm = new Array("小寒","大寒","立春","雨水","惊蛰","春分","清明","谷雨","立夏","小满","芒种","夏至","小暑","大暑","立秋","处暑","白露","秋分","寒露","霜降","立冬","小雪","大雪","冬至");
			var lFtv = new Array("0101*春节","0115 元宵节","0505 端午节","0707 七夕节","0715 中元节","0815 中秋节","0909 重阳节","1208 腊八节","1224 小年","0100*除夕");
			var sFtv = new Array("0101*元旦","0214 情人节","0308 妇女节","0312 植树节","0401 愚人节","0501 劳动节","0504 青年节","0512 护士节","0601 儿童节","0701 建党节","0801 建军节","0808 父亲节","0910 教师节","1001*国庆节","1006 老人节","1225 圣诞节");

			var sDObj = new Date(SY,SM,SD);
			var lDObj = new Lunar(sDObj);
			var lDPOS = new Array(3);
			var festival='',solarTerms='',solarFestival='',lunarFestival='',tmp1,tmp2;
			//农历节日 
			for(i in lFtv)
				if(lFtv[i].match(/^(\d{2})(.{2})([\s\*])(.+)$/)) {
					tmp1=Number(RegExp.$1)-lDObj.month;
					tmp2=Number(RegExp.$2)-lDObj.day;
					if(tmp1==0 && tmp2==0) lunarFestival=RegExp.$4; 
			}
			//国历节日
			for(i in sFtv)
				if(sFtv[i].match(/^(\d{2})(\d{2})([\s\*])(.+)$/)){
					tmp1=Number(RegExp.$1)-(SM+1);
					tmp2=Number(RegExp.$2)-SD;
					if(tmp1==0 && tmp2==0) solarFestival = RegExp.$4 ;
			}
			//节气
			  tmp1 = new Date((31556925974.7*(SY-1900)+sTermInfo[SM*2+1]*60000)+Date.UTC(1900,0,6,2,5));
			  tmp2 = tmp1.getUTCDate();
			  if (tmp2==SD) solarTerms = solarTerm[SM*2+1]  
			  tmp1 = new Date((31556925974.7*(SY-1900)+sTermInfo[SM*2]*60000)+Date.UTC(1900,0,6,2,5))
			  tmp2= tmp1.getUTCDate();
			  if (tmp2==SD) solarTerms = solarTerm[SM*2];

			  if(lunarFestival!='')
				  return lunarFestival; 
			  
			  if(solarFestival!='')
				  return solarFestival;

			  if(solarTerms!='')
				  return solarTerms;

			  festival=lDay(lDObj.day);
			  return festival;
		}
	}
}

/*************************************************************************
*
*			ThingsOS人机交互全局命名空间
*
**************************************************************************/
var TUI = {};
/*************************************************************************
*
*			定义命名空间的方法
*
**************************************************************************/
TUI.namespace = function (ns) {
    var a = arguments, o = null, i, j, d;
    for (i = 0; i < a.length; i = i + 1) {
        d = ("" + a[i]).split(".");
        o = TUI;

        // SmartOS is implied, so it is ignored if it is included
        for (j = (d[0] == "TUI") ? 1 : 0; j < d.length; j = j + 1) {
            o[d[j]] = o[d[j]] || {};
            o = o[d[j]];
        }
    }

    return o;
};

TUI.ns = TUI.namespace;

/*************************************************************************
*
*			继承对象类型的方法
*
**************************************************************************/
TUI.hasOwnProperty = (Object.prototype.hasOwnProperty) ?
    function (o, prop) {
        return o && o.hasOwnProperty && o.hasOwnProperty(prop);
    } : function (o, prop) {
        return !TUI.Utils.isUndefined(o[prop]) &&
                o.constructor.prototype[prop] !== o[prop];
    };

TUI.extend = function (subc, superc, overrides) {
    if (!superc || !subc) {
        throw new Error("extend failed, please check that " +
						"all dependencies are included.");
    }
    var f = function () { }, i;
    f.prototype = superc.prototype;
    subc.prototype = new f();
    subc.prototype.constructor = subc;
    subc.superclass = superc.prototype;
    if (superc.prototype.constructor == Object.prototype.constructor) {
        superc.prototype.constructor = superc;
    }

    if (overrides) {
        for (i in overrides) {
            if (TUI.hasOwnProperty(overrides, i)) {
                subc.prototype[i] = overrides[i];
            }
        }
        TUI.Utils._IEEnumFix(subc.prototype, overrides);
    }
};
/*************************************************************************
*
*			JSON处理的方法（encode，decode，toDate）
*
**************************************************************************/
TUI.JSON = new function () {
    this.decode = JSON.parse;
    this.encode = JSON.stringify;
	this.toString = function(){
			var	self = arguments.length ? arguments[0] : this,
				result, tmp;
			var tab=arguments.length>1 ?arguments[1]:"";
			var first=arguments.length>2 ?arguments[2]:0;
			if(self === null)
				result = "null";
			else if(self !== undefined && (tmp = $[typeof self](self))) {
				switch(tmp){
					case	Array:
						result = [];
						for(var	i = 0, j = 0, k = self.length; j < k; j++) {
							if(self[j] !== undefined && (tmp = this.encode(self[j],tab+"\t",i==0?1:0)))
								result[i++] = tmp;
						};
						result = "[".concat(result.join(",\r\n"),"]");
						break;
					case	Boolean:
						result = String(self);
						break;
					case	Date:
						result = '"'.concat(self.getFullYear(), '-', d(self.getMonth() + 1), '-', d(self.getDate()), 'T', d(self.getHours()), ':', d(self.getMinutes()), ':', d(self.getSeconds()), '"');
						break;
					case	Function:
						break;
					case	Number:
						result = isFinite(self) ? String(self) : "null";
						break;
					case	String:
						result = '"'.concat(self.replace(rs, s).replace(ru, u), '"');
						break;
					default:
						var	i = 0, key;
						result = [];
						for(key in self) {
							if(self[key] !== undefined && (tmp = this.encode(self[key],tab+"\t",1)))
								result[i++] = tab.concat('\t"',key.replace(rs, s).replace(ru, u), '":', tmp);
						};
						if(first==1)
							result = "{\r\n".concat(result.join(",\r\n"), "\r\n",tab,"}");
						else
							result = tab.concat("{\r\n",result.join(",\r\n"), "\r\n",tab,"}");
						break;
				}
			};
			return result;
		};

	var	c = {"\b":"b","\t":"t","\n":"n","\f":"f","\r":"r",'"':'"',"\\":"\\","/":"/"},
		d = function(n){return n<10?"0".concat(n):n},
		e = function(c,f,e){e=eval;delete eval;if(typeof eval==="undefined")eval=e;f=eval(""+c);eval=e;return f},
		i = function(e,p,l){return 1*e.substr(p,l)},
		p = ["","000","00","0",""],
		rc = null,
		rd = /^[0-9]{4}\-[0-9]{2}\-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}$/,
		rs = /(\x5c|\x2F|\x22|[\x0c-\x0d]|[\x08-\x0a])/g,
		rt = /^([0-9]+|[0-9]+[,\.][0-9]{1,3})$/,
		ru = /([\x00-\x07]|\x0b|[\x0e-\x1f])/g,
		s = function(i,d){return "\\".concat(c[d])},
		u = function(i,d){
			var	n=d.charCodeAt(0).toString(16);
			return "\\u".concat(p[n.length],n)
		},
		v = function(k,v){return $[typeof result](result)!==Function&&(v.hasOwnProperty?v.hasOwnProperty(k):v.constructor.prototype[k]!==v[k])},
		$ = {
			"boolean":function(){return Boolean},
			"function":function(){return Function},
			"number":function(){return Number},
			"object":function(o){return o instanceof o.constructor?o.constructor:null},
			"string":function(){return String},
			"undefined":function(){return null}
		},
		$$ = function(m){
			function $(c,t){t=c[m];delete c[m];try{e(c)}catch(z){c[m]=t;return 1}};
			return $(Array)&&$(Object)
		};
	try{rc=new RegExp('^("(\\\\.|[^"\\\\\\n\\r])*?"|[,:{}\\[\\]0-9.\\-+Eaeflnr-u \\n\\r\\t])+?$')}
	catch(z){rc=/^(true|false|null|\[.*\]|\{.*\}|".*"|\d+|\d+\.\d+)$/}
};

/*****************************************************************************************
*
*			浏览器端环境
*
*******************************************************************************************/
TUI.env = TUI.env || {};

/**
* parses a user agent string (or looks for one in navigator to parse if
* not supplied).
* @method parseUA
* @since 2.9.0
* @static
*/
TUI.env.parseUA = function (agent) {
var numberify = function (s) {
        var c = 0;
        return parseFloat(s.replace(/\./g, function () {
            return (c++ == 1) ? '' : '.';
        }));
    },

        nav = navigator,

        o = {

            /**
            * Internet Explorer version number or 0.  Example: 6
            * @property ie
            * @type float
            * @static
            */
            ie: 0,

            /**
            * Opera version number or 0.  Example: 9.2
            * @property opera
            * @type float
            * @static
            */
            opera: 0,

            /**
            * Gecko engine revision number.  Will evaluate to 1 if Gecko
            * is detected but the revision could not be found. Other browsers
            * will be 0.  Example: 1.8
            * <pre>
            * Firefox 1.0.0.4: 1.7.8   <-- Reports 1.7
            * Firefox 1.5.0.9: 1.8.0.9 <-- 1.8
            * Firefox 2.0.0.3: 1.8.1.3 <-- 1.81
            * Firefox 3.0   <-- 1.9
            * Firefox 3.5   <-- 1.91
            * </pre>
            * @property gecko
            * @type float
            * @static
            */
            gecko: 0,

            /**
            * AppleWebKit version.  KHTML browsers that are not WebKit browsers
            * will evaluate to 1, other browsers 0.  Example: 418.9
            * <pre>
            * Safari 1.3.2 (312.6): 312.8.1 <-- Reports 312.8 -- currently the
            *                                   latest available for Mac OSX 10.3.
            * Safari 2.0.2:         416     <-- hasOwnProperty introduced
            * Safari 2.0.4:         418     <-- preventDefault fixed
            * Safari 2.0.4 (419.3): 418.9.1 <-- One version of Safari may run
            *                                   different versions of webkit
            * Safari 2.0.4 (419.3): 419     <-- Tiger installations that have been
            *                                   updated, but not updated
            *                                   to the latest patch.
            * Webkit 212 nightly:   522+    <-- Safari 3.0 precursor (with native
            * SVG and many major issues fixed).
            * Safari 3.0.4 (523.12) 523.12  <-- First Tiger release - automatic
            * update from 2.x via the 10.4.11 OS patch.
            * Webkit nightly 1/2008:525+    <-- Supports DOMContentLoaded event.
            *                                   yahoo.com user agent hack removed.
            * </pre>
            * http://en.wikipedia.org/wiki/Safari_version_history
            * @property webkit
            * @type float
            * @static
            */
            webkit: 0,

            safari: 0,
            /**
            * Chrome will be detected as webkit, but this property will also
            * be populated with the Chrome version number
            * @property chrome
            * @type float
            * @static
            */
            chrome: 0,

            /**
            * The mobile property will be set to a string containing any relevant
            * user agent information when a modern mobile browser is detected.
            * Currently limited to Safari on the iPhone/iPod Touch, Nokia N-series
            * devices with the WebKit-based browser, and Opera Mini.
            * @property mobile
            * @type string
            * @static
            */
            mobile: null,

            /**
            * Adobe AIR version number or 0.  Only populated if webkit is detected.
            * Example: 1.0
            * @property air
            * @type float
            */
            air: 0,
            /**
            * Detects Apple iPad's OS version
            * @property ipad
            * @type float
            * @static
            */
            ipad: 0,
            /**
            * Detects Apple iPhone's OS version
            * @property iphone
            * @type float
            * @static
            */
            iphone: 0,
            /**
            * Detects Apples iPod's OS version
            * @property ipod
            * @type float
            * @static
            */
            ipod: 0,
            /**
            * General truthy check for iPad, iPhone or iPod
            * @property ios
            * @type float
            * @static
            */
            ios: null,
            /**
            * Detects Googles Android OS version
            * @property android
            * @type float
            * @static
            */
            android: 0,
            /**
            * Detects Palms WebOS version
            * @property webos
            * @type float
            * @static
            */
            webos: 0,

            /**
            * Google Caja version number or 0.
            * @property caja
            * @type float
            */
            caja: nav && nav.cajaVersion,

            /**
            * Set to true if the page appears to be in SSL
            * @property secure
            * @type boolean
            * @static
            */
            secure: false,
            /**
            *
            * support html5
            *
            */
            html5: false,
            /**
            *
            * support touch
            *
            */
            ontouch: false,
            /**
            *
            * support 3d
            *
            */
            has3d: false,

            /**
            *
            * clickEventType
            *
            */
            clickEventUp: "mouseup",
            clickEventDown: "mousedown",
            clickEventMove: "mousemove",
            /**
            * The operating system.  Currently only detecting windows or macintosh
            * @property os
            * @type string
            * @static
            */
            os: null

        },

    ua = agent || (navigator && navigator.userAgent),

    loc = window && window.location,

    href = loc && loc.href,

    m;

    o.secure = href && (href.toLowerCase().indexOf("https") === 0);
    o.html5 = !!document.createElement("canvas").getContext;
    o.ontouch = 'ontouchend' in document;
    o.has3d = 'WebKitCSSMatrix' in window && 'm11' in new WebKitCSSMatrix();

    if (window.navigator.msMaxTouchPoints>1) {
        o.clickEventUp = "pointerup";  //"MSPointerUp" for IE10;
        o.clickEventDown = "pointerdown";  //"MSPointerDown" for IE10;
        o.clickEventMove = "pointermove";  //"MSPointerMove" for IE10;
    }
    else {
        o.clickEventUp = (o.ontouch ? 'touchend' : 'mouseup');
        o.clickEventDown = (o.ontouch ? 'touchstart' : 'mousedown');
        o.clickEventMove = (o.ontouch ? 'touchmove' : 'mousemove');
    }

    if (ua) {

        if ((/windows|win32/i).test(ua)) {
            o.os = 'windows';
			//
			if (window.navigator.msMaxTouchPoints>1) {
				o.clickEventUp = "pointerup";  //"MSPointerUp" for IE10;
				o.clickEventDown = "pointerdown";  //"MSPointerDown" for IE10;
				o.clickEventMove = "pointermove";  //"MSPointerMove" for IE10;
			}
			else
			{
				o.ontouch=false;
				o.clickEventUp = 'mouseup';
				o.clickEventDown = 'mousedown';
				o.clickEventMove = 'mousemove';
			}
        } else if ((/macintosh/i).test(ua)) {
            o.os = 'macintosh';
        } else if ((/rhino/i).test(ua)) {
            o.os = 'rhino';
        }

        // Modern KHTML browsers should qualify as Safari X-Grade
        if ((/KHTML/).test(ua)) {
            o.webkit = 1;
        }
        // Modern WebKit browsers are at least X-Grade
        m = ua.match(/AppleWebKit([^\s]*)/);
        if (m && m[1]) {
            o.webkit = numberify(m[1]);

            // Mobile browser check
            if (/ Mobile\//.test(ua)) {
                o.mobile = 'Apple'; // iPhone or iPod Touch

                m = ua.match(/OS ([^\s]*)/);
                if (m && m[1]) {
                    m = numberify(m[1].replace('_', '.'));
                }
                o.ios = m;
                o.ipad = o.ipod = o.iphone = 0;

                m = ua.match(/iPad|iPod|iPhone/);
                if (m && m[0]) {
                    o[m[0].toLowerCase()] = o.ios;
                }
            } else {
                m = ua.match(/NokiaN[^\/]*|Android \d\.\d|webOS\/\d\.\d/);
                if (m) {
                    // Nokia N-series, Android, webOS, ex: NokiaN95
                    o.mobile = m[0];
                }
                if (/webOS/.test(ua)) {
                    o.mobile = 'WebOS';
                    m = ua.match(/webOS\/([^\s]*);/);
                    if (m && m[1]) {
                        o.webos = numberify(m[1]);
                    }
                }
                if (/Android/.test(ua) || /ipad; u;/.test(ua)) {
                    o.mobile = 'Android';
                    m = ua.match(/Android ([^\s]*);/);
                    if (m && m[1]) {
                        o.android = numberify(m[1]);
                    }

                }
            }

            m = ua.match(/Chrome\/([^\s]*)/);
            if (m && m[1]) {
                o.chrome = numberify(m[1]); // Chrome
            } else {
                m = ua.match(/AdobeAIR\/([^\s]*)/);
                if (m) {
                    o.air = m[0]; // Adobe AIR 1.0 or better
                }
                else {
                    m = ua.match(/Safari\/([^\s]*)/);
                    if (m) {
                        o.safari = m[0];
                    }
                }
            }
        }

        if (!o.webkit) { // not webkit
            // @todo check Opera/8.01 (J2ME/MIDP; Opera Mini/2.0.4509/1316; fi; U; ssr)
            m = ua.match(/Opera[\s\/]([^\s]*)/);
            if (m && m[1]) {
                o.opera = numberify(m[1]);
                m = ua.match(/Version\/([^\s]*)/);
                if (m && m[1]) {
                    o.opera = numberify(m[1]); // opera 10+
                }
                m = ua.match(/Opera Mini[^;]*/);
                if (m) {
                    o.mobile = m[0]; // ex: Opera Mini/2.0.4509/1316
                }
            } else { // not opera or webkit
                m = ua.match(/MSIE\s([^;]*)/);
                if (m && m[1]) {
                    o.ie = numberify(m[1]);
                } else { // not opera, webkit, or ie
                    m = ua.match(/Trident\/([^\s]*)/);
					if (m) {
						o.ie = 11;
						m = ua.match(/rv:([^\s\)]*)/);
						if (m && m[1]) {
							o.ie = numberify(m[1]);
						}
					}
					else
                    {
						m = ua.match(/Gecko\/([^\s]*)/);
						if (m) {
							o.gecko = 1; // Gecko detected, look for revision
							m = ua.match(/rv:([^\s\)]*)/);
							if (m && m[1]) {
								o.gecko = numberify(m[1]);
							}
						}
					}
                }
            }
        }
    }
    return o;
};

TUI.env.ua = TUI.env.parseUA();
TUI.env.log = [];
TUI.env.us = null;
TUI.env.os = [];
TUI.env.cfg = null;
TUI.env.desktop = 0;
TUI.env.login = null;

if (window != top) {
    TUI.env.us = top.TUI.env.us;
    TUI.env.log = top.TUI.env.log;
    TUI.env.os = top.TUI.env.os;
    TUI.env.cfg = top.TUI.env.cfg;
    TUI.env.desktop = top.TUI.env.desktop;
}
/*****************************************************************************************
*
*			扩展方法工具包
*
*******************************************************************************************/
TUI.Utils = new function () {
    var keyStr = "ABCDEFGHIJKLMNOP" +
                "QRSTUVWXYZabcdef" +
                "ghijklmnopqrstuv" +
                "wxyz0123456789+/" +
                "=",
    OP = Object.prototype,
	ARRAY_TOSTRING = '[object Array]',
    FUNCTION_TOSTRING = '[object Function]',
    OBJECT_TOSTRING = '[object Object]',
    NOTHING = [],

    HTML_CHARS = {
        '&': '&amp;',
        '<': '&lt;',
        '>': '&gt;',
        '"': '&quot;',
        "'": '&#x27;',
        '/': '&#x2F;',
        '`': '&#x60;'
    },

    // ADD = ["toString", "valueOf", "hasOwnProperty"],
    ADD = ["toString", "valueOf"];

    /* Section: Methods - Public */
    /**
    * Determines wheather or not the provided object is an array.
    * @method isArray
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isArray = function (o) {
        return OP.toString.apply(o) === ARRAY_TOSTRING;
    };

    /**
    * Determines whether or not the provided object is a boolean
    * @method isBoolean
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isBoolean = function (o) {
        return typeof o === 'boolean';
    };

    /**
    * Determines whether or not the provided object is a function.
    * Note: Internet Explorer thinks certain functions are objects:
    *
    * var obj = document.createElement("object");
    * TUI.lang.isFunction(obj.getAttribute) // reports false in IE
    *
    * var input = document.createElement("input"); // append to body
    * TUI.lang.isFunction(input.focus) // reports false in IE
    *
    * You will have to implement additional tests if these functions
    * matter to you.
    *
    * @method isFunction
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isFunction = function (o) {
        return (typeof o === 'function') || OP.toString.apply(o) === FUNCTION_TOSTRING;
    };

    /**
    * Determines whether or not the provided object is null
    * @method isNull
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isNull = function (o) {
        return o === null;
    };

    /**
    * Determines whether or not the provided object is a legal number
    * @method isNumber
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isNumber = function (o) {
        return typeof o === 'number' && isFinite(o);
    };

    /**
    * Determines whether or not the provided object is of type object
    * or function
    * @method isObject
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isObject = function (o) {
        return (o && (typeof o === 'object' || this.isFunction(o))) || false;
    };

    /**
    * Determines whether or not the provided object is a string
    * @method isString
    * @param {any} o The object being testing
    * @return {boolean} the result
    */
    this.isString = function (o) {
        return typeof o === 'string';
    };

    this.isUndefined = function (o) {
        return typeof o === 'undefined';
    };

    /**
    * Returns a string without any leading or trailing whitespace.  If
    * the input is not a string, the input will be returned untouched.
    * @method trim
    * @since 2.3.0
    * @param s {string} the string to trim
    * @return {string} the trimmed string
    */
    this.trim = function (s) {
        try {
            return s.replace(/^\s+|\s+$/g, "");
        } catch (e) {
            return s;
        }
    };

    /**
    * A convenience method for detecting a legitimate non-null value.
    * Returns false for null/undefined/NaN, true for other values,
    * including 0/false/''
    * @method isValue
    * @since 2.3.0
    * @param o {any} the item to test
    * @return {boolean} true if it is not null/undefined/NaN || false
    */
    this.isValue = function (o) {
        // return (o || o === false || o === 0 || o === ''); // Infinity fails
        return (this.isObject(o) || this.isString(o) || this.isNumber(o) || this.isBoolean(o));
    };

    //校验是否全由数字组成 
    this.isDigit = function (s) {
        var patrn = /^[0-9]{1,20}$/;
        if (!patrn.exec(s))
            return false;

        return true;
    }

    //校验密码：只能输入6-20个字母、数字、下划线  
    this.isPasswd = function (s) {
        var patrn = /^(\w){6,20}$/;
        if (!patrn.exec(s))
            return false;
        return true;
    }

    //校验普通电话、传真号码：可以“+”开头，除数字外，可含有“-”  
    this.isTel = function (s) {
        var patrn = /^[+]{0,1}(\d){1,3}[ ]?([-]?((\d)|[ ]){1,12})+$/;
        if (!patrn.exec(s))
            return false;
        return true;
    }

    //校验手机号码：必须以数字开头，除数字外，可含有“-”  
    this.isMobil = function (s) {
        var patrn = /^[+]{0,1}(\d){1,3}[ ]?([-]?((\d)|[ ]){1,12})+$/;
        if (!patrn.exec(s))
            return false;
        return true;
    }


    this.isIPAddr = function (s) //by zergling  
    {
        var patrn = /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])(\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])){3}$/;
        if (!patrn.exec(s))
            return false;
        return true;
    }

    /**
    * IE will not enumerate native functions in a derived object even if the
    * function was overridden.  This is a workaround for specific functions
    * we care about on the Object prototype.
    * @property _IEEnumFix
    * @param {Function} r  the object to receive the augmentation
    * @param {Function} s  the object that supplies the properties to augment
    * @static
    * @private
    */
    this._IEEnumFix = (TUI.env.ua.ie) ? function (r, s) {
        var i, fname, f;
        for (i = 0; i < ADD.length; i = i + 1) {

            fname = ADD[i];
            f = s[fname];

            if (this.isFunction(f) && f != OP[fname]) {
                r[fname] = f;
            }
        }
    } : function () { };
	/*
    Method: parseDate
    transforms a encoded Date string into a native Date object.
	
    Arguments:
    [String/Number] - Optional Date string or server time if this method is not a String prototype. Server time should be an integer, based on seconds since 1970/01/01 or milliseconds / 1000 since 1970/01/01.
	
    Returns:
    Date - Date object or undefined if string is not a valid Date
	
    Example [Basic]:
    >var	serverDate = Utils.parseDate("2007-04-05 08:36:46");
    >alert(serverDate.getMonth());	// 3 (months start from 0)

    */

    this.parseDate = function (str) {
        if (typeof str == 'string') {
            var results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) *$/);
            if (results && results.length > 3)
                return new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}) *$/);
            if (results && results.length > 5)
                return new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}):(\d{1,2}) *$/);
            if (results && results.length > 6)
                return new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10), parseInt(results[6], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}):(\d{1,2})\.(\d{1,9}) *$/);
            if (results && results.length > 7)
                return new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10), parseInt(results[6], 10), parseInt(results[7], 10));
        }
        else if (typeof str == 'number') {
            return new Date(str * 1000);
        }
        return null;
    };

	this.dateDiff = function (interval,startTime, endTime) {
            switch (interval) {
                case "s":
                    return parseInt((endTime - startTime) / 1000);
                case "n":
                    return parseInt((endTime - startTime) / 60000);
                case "h":
                    return parseInt((endTime - startTime) / 3600000);
                case "d":
                    return parseInt((endTime - startTime) / 86400000);
                case "w":
                    return parseInt((endTime - startTime) / (86400000 * 7));
                case "m":
                    return (endTime.getMonth() + 1) + ((endTime.getFullYear() - startTime.getFullYear()) * 12) - (startTime.getMonth() + 1);
                case "y":
                    return endTime.getFullYear() - startTime.getFullYear();
                default:
                    return undefined;
            }
	};

	this.dateMessage= function (str) {
		var now=new Date();
		var tc=new Date(now.getFullYear(),now.getMonth(),now.getDate(),0,0,0,0);	//今天
		var t1=tc.DateAdd("d",-1);	//昨天
		var t2=tc.DateAdd("d",-2);	//前天
		var t3=new Date(now.getFullYear(),0,1,0,0,0,0);	//今年
		var t4=tc.DateAdd("d",1);	//明天
		//
		var time=str;
		if (typeof str == 'string') {
            var results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) *$/);
            if (results && results.length > 3)
                time = new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}) *$/);
            if (results && results.length > 5)
                time = new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}):(\d{1,2}) *$/);
            if (results && results.length > 6)
                time = new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10), parseInt(results[6], 10));
            results = str.match(/^ *(\d{4})-(\d{1,2})-(\d{1,2}) +(\d{1,2}):(\d{1,2}):(\d{1,2})\.(\d{1,9}) *$/);
            if (results && results.length > 7)
                time = new Date(parseInt(results[1], 10), parseInt(results[2], 10) - 1, parseInt(results[3], 10), parseInt(results[4], 10), parseInt(results[5], 10), parseInt(results[6], 10), parseInt(results[7], 10));
        }
        else if (typeof str == 'number') {
            time = new Date(str * 1000);
        }
		//
		if(time>t4)
		{
			return time.Format("yyyy年MM月");
		}
		else if(time>tc)
		{
			return time.Format("今天 hh:mm");
		}
		else if(time>t1)
		{
			return time.Format("昨天 hh:mm");
		}
		else if(time>t2)
		{
			return time.Format("前天 hh:mm");
		}
		else if(time>t3)
		{
			return time.Format("MM月dd日");
		}
		else
		{
			return time.Format("yyyy年MM月");
		}
	};
    //输出调试日志
    this.log = function (msg, cat, src) {
        if (TUI.Utils.isUndefined(msg))
            return;
        if (TUI.Utils.isUndefined(cat))
            cat = "info";
        if (TUI.Utils.isUndefined(src))
            src = window.location.pathname;
        //
        var tc = new Date();
        if (TUI.env.log.length > 1024) {
            TUI.env.log.shift();
        }
        TUI.env.log.push({ msg: tc.Format("hh:mm:ss.S") + " " + msg, cat: cat, src: src, type: "text" });
        //
    };
    //Base64编码
    this.encode64 = function (input) {
        input = escape(input).replace(/\+/g, '%2B');
        var output = "";
        var chr1, chr2, chr3 = "";
        var enc1, enc2, enc3, enc4 = "";
        var i = 0;

        do {
            chr1 = input.charCodeAt(i++);
            chr2 = input.charCodeAt(i++);
            chr3 = input.charCodeAt(i++);

            enc1 = chr1 >> 2;
            enc2 = ((chr1 & 3) << 4) | (chr2 >> 4);
            enc3 = ((chr2 & 15) << 2) | (chr3 >> 6);
            enc4 = chr3 & 63;

            if (isNaN(chr2)) {
                enc3 = enc4 = 64;
            } else if (isNaN(chr3)) {
                enc4 = 64;
            }

            output = output +
			keyStr.charAt(enc1) +
			keyStr.charAt(enc2) +
			keyStr.charAt(enc3) +
			keyStr.charAt(enc4);
            chr1 = chr2 = chr3 = "";
            enc1 = enc2 = enc3 = enc4 = "";
        } while (i < input.length);

        return output;
    };
    //Base64解码
    this.decode64 = function (input) {
        var output = "";
        var chr1, chr2, chr3 = "";
        var enc1, enc2, enc3, enc4 = "";
        var i = 0;

        // remove all characters that are not A-Z, a-z, 0-9, +, /, or =
        var base64test = /[^A-Za-z0-9\+\/\=]/g;
        if (base64test.exec(input)) {
            alert("There were invalid base64 characters in the input text.\n" +
			   "Valid base64 characters are A-Z, a-z, 0-9, '+', '/', and '='\n" +
			   "Expect errors in decoding.");
        }


        input = input.replace(/[^A-Za-z0-9\+\/\=]/g, "");


        do {
            enc1 = keyStr.indexOf(input.charAt(i++));
            enc2 = keyStr.indexOf(input.charAt(i++));
            enc3 = keyStr.indexOf(input.charAt(i++));
            enc4 = keyStr.indexOf(input.charAt(i++));

            chr1 = (enc1 << 2) | (enc2 >> 4);
            chr2 = ((enc2 & 15) << 4) | (enc3 >> 2);
            chr3 = ((enc3 & 3) << 6) | enc4;

            output = output + String.fromCharCode(chr1);

            if (enc3 != 64) {
                output = output + String.fromCharCode(chr2);
            }
            if (enc4 != 64) {
                output = output + String.fromCharCode(chr3);
            }

            chr1 = chr2 = chr3 = "";
            enc1 = enc2 = enc3 = enc4 = "";

        } while (i < input.length);


        return unescape(output);
    };

    /**
    * 数字对象的格式化;
    */
    this.formatNumber = function (num, pattern) {
        var strarr = num ? num.toString().split('.') : ['0'];
        var fmtarr = pattern ? pattern.split('.') : [''];
        var retstr = '';

        // 整数部分  
        var str = strarr[0];
        var fmt = fmtarr[0];
        var i = str.length - 1;
        var comma = false;
        for (var f = fmt.length - 1; f >= 0; f--) {
            switch (fmt.substr(f, 1)) {
                case '#':
                    if (i >= 0) retstr = str.substr(i--, 1) + retstr;
                    break;
                case '0':
                    if (i >= 0) retstr = str.substr(i--, 1) + retstr;
                    else retstr = '0' + retstr;
                    break;
                case ',':
                    comma = true;
                    retstr = ',' + retstr;
                    break;
            }
        }
        if (i >= 0) {
            if (comma) {
                var l = str.length;
                for (; i >= 0; i--) {
                    retstr = str.substr(i, 1) + retstr;
                    if (i > 0 && ((l - i) % 3) == 0) retstr = ',' + retstr;
                }
            }
            else retstr = str.substr(0, i + 1) + retstr;
        }

        retstr = retstr + '.';
        // 处理小数部分  
        str = strarr.length > 1 ? strarr[1] : '';
        fmt = fmtarr.length > 1 ? fmtarr[1] : '';
        i = 0;
        for (var f = 0; f < fmt.length; f++) {
            switch (fmt.substr(f, 1)) {
                case '#':
                    if (i < str.length) retstr += str.substr(i++, 1);
                    break;
                case '0':
                    if (i < str.length) retstr += str.substr(i++, 1);
                    else retstr += '0';
                    break;
            }
        }
        return retstr.replace(/^,+/, '').replace(/\.$/, '');
    };
	/*
	* 使用最小二乘法计算斜率
	*/
	this.computerSlope = function (xyList)
	{
		var m=0;
		var nCount=0;
		var SumX, SumY, SumXY, SumX2;
		//
		SumX = 0;
		SumX2 = 0;
		for(var i=0; i<xyList.length; i++)
		{
			if(xyList[i][0]!=null && xyList[i][1]!=null)
			{
				SumX += xyList[i][0];
				SumX2 += (xyList[i][0] * xyList[i][0]);
				nCount++;
			}
		}
	 
		SumY = 0;
		SumXY = 0;
		for(var i=0; i<xyList.length; i++)
		{
			if(xyList[i][0]!=null && xyList[i][1]!=null)
			{
				SumY += xyList[i][1];
				SumXY += ( xyList[i][0] * xyList[i][1]);
			}
		}
	 
		m = ((nCount*SumXY - SumX*SumY) / (nCount*SumX2 - SumX * SumX));
		//
		return m;
	}
	/**
    * 精确浮点数精度;
    */
    this.floatPrecision = function (src, pos)
	{
		return Math.round(src*Math.pow(10, pos))/Math.pow(10, pos);
	};
	/**
    * 解析布尔值
    */
	this.parseBoolean = function (string) {
	  switch (String(string).toLowerCase()) {
		case "true":
		case "1":
		case "yes":
		case "y":
		  return true;
		case "false":
		case "0":
		case "no":
		case "n":
		  return false;
		default:
		  //you could throw an error, but 'undefined' seems a more logical reply
		  return undefined;
	  }
	};

	/*
    *    从0开始取n个点位路径名称
    */
	this.getLeftPathTag = function(strFullPathName, nCount) {
		var arr = strFullPathName.split(".");
		var res = [];
		for (l = 0; l < nCount; l++) {
			res[res.length] = arr[l];
		}
		return res.join(".");
	};
	/*
    *   从nLeft开始取后续点位路径名称
    */
	this.getRightPathName = function(strFullPathName, nLeft) {
		var arr = strFullPathName.split(".");
		var res = [];
		for (l = nLeft; l < arr.length; l++) {
			res[res.length] = arr[l];
		}
		return res.join(".");
	};

	this.playUserSound = function(serverId){
		if(TUI.env.wx!=undefined
			&& TUI.env.wx.ready==true)
		{
			wx.downloadVoice({
				serverId: serverId, 
				isShowProgressTips: 0,
				success: function (res) {
					wx.playVoice({
						localId: res.localId
					});
				}
			});
		}
	};

    //计算字符串Hash值
    var hexcase = 0;
    var b64pad = "";
    var chrsz = 8;
    this.hex_sha1 = function (s) { return binb2hex(core_sha1(str2binb(s), s.length * chrsz)); }
    this.hex_hmac_sha1 = function (key, data) { return binb2hex(core_hmac_sha1(key, data)); }

    function core_sha1(x, len) {
        /* append padding */
        x[len >> 5] |= 0x80 << (24 - len % 32);
        x[((len + 64 >> 9) << 4) + 15] = len;

        var w = Array(80);
        var a = 1732584193;
        var b = -271733879;
        var c = -1732584194;
        var d = 271733878;
        var e = -1009589776;

        for (var i = 0; i < x.length; i += 16) {
            var olda = a;
            var oldb = b;
            var oldc = c;
            var oldd = d;
            var olde = e;

            for (var j = 0; j < 80; j++) {
                if (j < 16) w[j] = x[i + j];
                else w[j] = rol(w[j - 3] ^ w[j - 8] ^ w[j - 14] ^ w[j - 16], 1);
                var t = safe_add(safe_add(rol(a, 5), sha1_ft(j, b, c, d)),
                       safe_add(safe_add(e, w[j]), sha1_kt(j)));
                e = d;
                d = c;
                c = rol(b, 30);
                b = a;
                a = t;
            }

            a = safe_add(a, olda);
            b = safe_add(b, oldb);
            c = safe_add(c, oldc);
            d = safe_add(d, oldd);
            e = safe_add(e, olde);
        }
        return Array(a, b, c, d, e);

    }
    function sha1_ft(t, b, c, d) {
        if (t < 20) return (b & c) | ((~b) & d);
        if (t < 40) return b ^ c ^ d;
        if (t < 60) return (b & c) | (b & d) | (c & d);
        return b ^ c ^ d;
    }
    function sha1_kt(t) {
        return (t < 20) ? 1518500249 : (t < 40) ? 1859775393 :
         (t < 60) ? -1894007588 : -899497514;
    }
    function core_hmac_sha1(key, data) {
        var bkey = str2binb(key);
        if (bkey.length > 16) bkey = core_sha1(bkey, key.length * chrsz);

        var ipad = Array(16), opad = Array(16);
        for (var i = 0; i < 16; i++) {
            ipad[i] = bkey[i] ^ 0x36363636;
            opad[i] = bkey[i] ^ 0x5C5C5C5C;
        }

        var hash = core_sha1(ipad.concat(str2binb(data)), 512 + data.length * chrsz);
        return core_sha1(opad.concat(hash), 512 + 160);
    }
    function safe_add(x, y) {
        var lsw = (x & 0xFFFF) + (y & 0xFFFF);
        var msw = (x >> 16) + (y >> 16) + (lsw >> 16);
        return (msw << 16) | (lsw & 0xFFFF);
    }
    function rol(num, cnt) {
        return (num << cnt) | (num >>> (32 - cnt));
    }
    function str2binb(str) {
        var bin = Array();
        var mask = (1 << chrsz) - 1;
        for (var i = 0; i < str.length * chrsz; i += chrsz)
            bin[i >> 5] |= (str.charCodeAt(i / chrsz) & mask) << (32 - chrsz - i % 32);
        return bin;
    }
    function binb2hex(binarray) {
        var hex_tab = hexcase ? "0123456789ABCDEF" : "0123456789abcdef";
        var str = "";
        for (var i = 0; i < binarray.length * 4; i++) {
            str += hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8 + 4)) & 0xF) +
           hex_tab.charAt((binarray[i >> 2] >> ((3 - i % 4) * 8)) & 0xF);
        }
        return str;
    }

    /*
    Method: varDump
    print a native Date object to string.
	
    Arguments:
    [object data,addwhitespace,safety,level].
	
    Returns:
    String or Html
	
    Example:
    >var aa="sddd";
    >var bb={a:1,b:34.5,df:aa};
    >var cc={x:bb,aa:1};
    >alert(Utils.varDump(cc,"html",5));

    */
    this.dump = function (object, cat, src) {
        if (TUI.Utils.isUndefined(cat))
            cat = "info";
        if (TUI.Utils.isUndefined(src))
            src = window.location.pathname;
        //
        var tc = new Date();
        if (TUI.env.log.length > 1024) {
            TUI.env.log.shift();
        }
        //
        var st = typeof showTypes == 'undefined' ? true : showTypes;
        var dumpHtml = (/string|number|undefined|boolean/.test(typeof (object)) || object == null) ? object : recurse(object, typeof object);
        //
        TUI.env.log.push({ msg: dumpHtml, cat: cat, src: tc.Format("hh:mm:ss.S") + " " + cat + " [" + src + "]", type: "html" });
        //
        function recurse(o, type) {
            var i;
            var j = 0;
            var r = '';
            type = _dumpType(o);
            switch (type) {
                case 'regexp':
                    var t = type;
                    r += '<table' + _dumpStyles(t, 'table') + '><tr><th colspan="2"' + _dumpStyles(t, 'th') + '>' + t + '</th></tr>';
                    r += '<tr><td colspan="2"' + _dumpStyles(t, 'td-value') + '><table' + _dumpStyles('arguments', 'table') + '><tr><td' + _dumpStyles('arguments', 'td-key') + '><i>RegExp: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o + '</td></tr></table>';
                    j++;
                    break;
                case 'date':
                    var t = type;
                    r += '<table' + _dumpStyles(t, 'table') + '><tr><th colspan="2"' + _dumpStyles(t, 'th') + '>' + t + '</th></tr>';
                    r += '<tr><td colspan="2"' + _dumpStyles(t, 'td-value') + '><table' + _dumpStyles('arguments', 'table') + '><tr><td' + _dumpStyles('arguments', 'td-key') + '><i>Date: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o + '</td></tr></table>';
                    j++;
                    break;
                case 'function':
                    var t = type;
                    var a = o.toString().match(/^.*function.*?\((.*?)\)/im);
                    var args = (a == null || typeof a[1] == 'undefined' || a[1] == '') ? 'none' : a[1];
                    r += '<table' + _dumpStyles(t, 'table') + '><tr><th colspan="2"' + _dumpStyles(t, 'th') + '>' + t + '</th></tr>';
                    r += '<tr><td colspan="2"' + _dumpStyles(t, 'td-value') + '><table' + _dumpStyles('arguments', 'table') + '><tr><td' + _dumpStyles('arguments', 'td-key') + '><i>Arguments: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + args + '</td></tr><tr><td' + _dumpStyles('arguments', 'td-key') + '><i>Function: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o + '</td></tr></table>';
                    j++;
                    break;
                case 'domelement':
                    var t = type;
                    r += '<table' + _dumpStyles(t, 'table') + '><tr><th colspan="2"' + _dumpStyles(t, 'th') + '>' + t + '</th></tr>';
                    r += '<tr><td' + _dumpStyles(t, 'td-key') + '><i>Node Name: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o.nodeName.toLowerCase() + '</td></tr>';
                    r += '<tr><td' + _dumpStyles(t, 'td-key') + '><i>Node Type: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o.nodeType + '</td></tr>';
                    r += '<tr><td' + _dumpStyles(t, 'td-key') + '><i>Node Value: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o.nodeValue + '</td></tr>';
                    r += '<tr><td' + _dumpStyles(t, 'td-key') + '><i>innerHTML: </i></td><td' + _dumpStyles(type, 'td-value') + '>' + o.innerHTML + '</td></tr>';
                    j++;
                    break;
            }
            if (/object|array/.test(type)) {
                for (i in o) {
                    var t = _dumpType(o[i]);
                    if (j < 1) {
                        r += '<table' + _dumpStyles(type, 'table') + '><tr><th colspan="2"' + _dumpStyles(type, 'th') + '>' + type + '</th></tr>';
                        j++;
                    }
                    if (typeof o[i] == 'object' && o[i] != null) {
                        r += '<tr><td' + _dumpStyles(type, 'td-key') + '>' + i + (st ? ' [' + t + ']' : '') + '</td><td' + _dumpStyles(type, 'td-value') + '>' + recurse(o[i], t) + '</td></tr>';
                    } else if (typeof o[i] == 'function') {
                        r += '<tr><td' + _dumpStyles(type, 'td-key') + '>' + i + (st ? ' [' + t + ']' : '') + '</td><td' + _dumpStyles(type, 'td-value') + '>' + recurse(o[i], t) + '</td></tr>';
                    } else {
                        r += '<tr><td' + _dumpStyles(type, 'td-key') + '>' + i + (st ? ' [' + t + ']' : '') + '</td><td' + _dumpStyles(type, 'td-value') + '>' + o[i] + '</td></tr>';
                    }
                }
            }
            if (j == 0) {
                r += '<table' + _dumpStyles(type, 'table') + '><tr><th colspan="2"' + _dumpStyles(type, 'th') + '>' + type + ' [empty]</th></tr>';
            }
            r += '</table>';
            return r;
        };

        function _dumpStyles(type, use) {
            var r = '';
            var table = 'font-size:xx-small;font-family:verdana,arial,helvetica,sans-serif;cell-spacing:2px;';
            var th = 'font-size:xx-small;font-family:verdana,arial,helvetica,sans-serif;text-align:left;color: white;padding: 5px;vertical-align :top;cursor:hand;cursor:pointer;';
            var td = 'font-size:xx-small;font-family:verdana,arial,helvetica,sans-serif;vertical-align:top;padding:3px;';
            var thScript = 'onClick="tTable(this);" title="click to collapse"';
            var tdScript = 'onClick="tRow(this);" title="click to collapse"';
            switch (type) {
                case 'string':
                case 'number':
                case 'boolean':
                case 'undefined':
                case 'object':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#0000cc;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#4444cc;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + td + 'background-color:#ccddff;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
                case 'array':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#006600;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#009900;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + td + 'background-color:#ccffcc;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
                case 'function':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#aa4400;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#cc6600;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + td + 'background-color:#fff;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
                case 'arguments':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#dddddd;cell-spacing:3;"';
                            break;
                        case 'td-key':
                            r = ' style="' + th + 'background-color:#eeeeee;color:#000000;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                    }
                    break;
                case 'regexp':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#CC0000;cell-spacing:3;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#FF0000;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + th + 'background-color:#FF5757;color:#000000;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
                case 'date':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#663399;cell-spacing:3;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#9966CC;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + th + 'background-color:#B266FF;color:#000000;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
                case 'domelement':
                    switch (use) {
                        case 'table':
                            r = ' style="' + table + 'background-color:#FFCC33;cell-spacing:3;"';
                            break;
                        case 'th':
                            r = ' style="' + th + 'background-color:#FFD966;"' + thScript;
                            break;
                        case 'td-key':
                            r = ' style="' + th + 'background-color:#FFF2CC;color:#000000;cursor:hand;cursor:pointer;"' + tdScript;
                            break;
                        case 'td-value':
                            r = ' style="' + td + 'background-color:#fff;"';
                            break;
                    }
                    break;
            }
            return r;
        };
        function _dumpType(obj) {
            var t = typeof (obj);
            if (t == 'function') {
                var f = obj.toString();
                if ((/^\/.*\/[gi]??[gi]??$/).test(f)) {
                    return 'regexp';
                } else if ((/^\[object.*\]$/i).test(f)) {
                    t = 'object'
                }
            }
            if (t != 'object') {
                return t;
            }
            switch (obj) {
                case null:
                    return 'null';
                case window:
                    return 'window';
                case document:
                    return document;
                case window.event:
                    return 'event';
            }
            if (window.event && (event.type == obj.type)) {
                return 'event';
            }
            var c = obj.constructor;
            if (c != null) {
                switch (c) {
                    case Array:
                        t = 'array';
                        break;
                    case Date:
                        return 'date';
                    case RegExp:
                        return 'regexp';
                    case Object:
                        t = 'object';
                        break;
                    case ReferenceError:
                        return 'error';
                    default:
                        var sc = c.toString();
                        var m = sc.match(/\s*function (.*)\(/);
                        if (m != null) {
                            return 'object';
                        }
                }
            }
            var nt = obj.nodeType;
            if (nt != null) {
                switch (nt) {
                    case 1:
                        if (obj.item == null) {
                            return 'domelement';
                        }
                        break;
                    case 3:
                        return 'string';
                }
            }
            if (obj.toString != null) {
                var ex = obj.toString();
                var am = ex.match(/^\[object (.*)\]$/i);
                if (am != null) {
                    var am = am[1];
                    switch (am.toLowerCase()) {
                        case 'event':
                            return 'event';
                        case 'nodelist':
                        case 'htmlcollection':
                        case 'elementarray':
                            return 'array';
                        case 'htmldocument':
                            return 'htmldocument';
                    }
                }
            }
            return t;
        };
    };
};

TUI.Comet = new function () {
    var now = new Date();
    var cometTask = [];
    var tunnelID = Math.floor(now.getTime() / 1000);
    var cometID = 0;
    //
    var handle = this;
    var dataObject = new XMLHttpRequest();
    var eventObject = new XMLHttpRequest();
    //
    this.errorText = "";
    this.cometStatus = "close";
    this.taskCount = 0;
    //
    this.connect = function () {
        if (eventObject) {
            if (handle.cometStatus != "close") {
                return true;
            }

            tunnelID ++;

            try {
                eventObject.open("GET",  $api.getStorage('thingslabs-url')+"/WebMobile/cometevent.srv?CmdType=Connect&TunnelID=" + tunnelID + "&CometID=0&ReqType=all&ReqStr=all", false);
                //
                eventObject.setRequestHeader("If-Modified-Since", "0");
                eventObject.send(null);
                //
                var cometStatusCode = parseInt((eventObject.responseXML.documentElement.attributes["StatusCode"]==undefined)?eventObject.responseXML.documentElement.attributes[0].value:eventObject.responseXML.documentElement.attributes["StatusCode"].value);
                if (cometStatusCode == 7) {
                    handle.cometStatus = "connect";
                    return true;
                }
                else {
                    handle.errorText = decodeURIComponent(eventObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' ');
                }
            }
            catch (ex) {
                handle.errorText = ex.message;
            }
        }
        handle.close();
        return false;
    };
    //
    this.onlisten = function () {
        if (handle.cometStatus == "onlisten") {
            return true;
        }

        try {
            dataObject.open("GET", $api.getStorage('thingslabs-url')+"/WebMobile/cometevent.srv?CmdType=Listen&TunnelID=" + tunnelID + "&CometID=0&ReqType=all&ReqStr=all", true);
            dataObject.setRequestHeader("If-Modified-Since", "0");
            //
            dataObject.onreadystatechange = function () {
                if (dataObject.readyState == 4 && dataObject.status == 200) {
                    if (dataObject.responseText == "")//防止防火墙掐断长连接
                    {
                        handle.onlisten();
                        return;
                    }
                    //
                    var cometTunnelID = parseInt((dataObject.responseXML.documentElement.attributes["TunnelID"]==undefined)?dataObject.responseXML.documentElement.attributes[2].value:dataObject.responseXML.documentElement.attributes["TunnelID"].value);
                    if (cometTunnelID != tunnelID) {
						handle.errorText = "Comet侦听数据异常";
                        handle.close();
                        return;
                    }
                    //
                    var cometStatusCode = parseInt((dataObject.responseXML.documentElement.attributes["StatusCode"]==undefined)?dataObject.responseXML.documentElement.attributes[0].value:dataObject.responseXML.documentElement.attributes["StatusCode"].value);
                    if (cometStatusCode == 4) {
                        handle.cometStatus = "data";
                        for (i = 0; i < cometTask.length; i++) {
                            if (cometTask[i].cometID == parseInt((dataObject.responseXML.documentElement.attributes["CometID"]==undefined)?dataObject.responseXML.documentElement.attributes[3].value:dataObject.responseXML.documentElement.attributes["CometID"].value)) {
                                handle.onchange = cometTask[i].fn;
                                if (typeof handle.onchange == "function") {
                                    handle.onchange(decodeURIComponent(dataObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' '));
                                }
                                break;
                            }
                        }
                        //
                        handle.onlisten();
                    }
                    else if (cometStatusCode == 2) {
                        for (i = 0; i < cometTask.length; i++) {
                            if (cometTask[i].cometID == parseInt((dataObject.responseXML.documentElement.attributes["CometID"]==undefined)?dataObject.responseXML.documentElement.attributes[3].value:dataObject.responseXML.documentElement.attributes["CometID"].value)) {
                                handle.onchange = cometTask[i].fn;
                                if (typeof handle.onchange == "function") {
                                    handle.onchange(decodeURIComponent(dataObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' '));
                                }
                                //
                                handle.cometStatus = "leave";
                                handle.taskCount--;
                                cometTask.splice(i, 1);
                                break;
                            }
                        }
                        //
                        handle.onlisten();
                    }
                    else if (cometStatusCode == 5) {
                        handle.cometStatus = "close";
                        handle.taskCount = 0;
                        cometTask = [];
                    }
                    else {
                        handle.errorText = "Comet侦听通讯异常";
                        handle.close();
                    }
                }
            };
            dataObject.send(null);
            handle.cometStatus = "onlisten";
            return true;
        }
        catch (ex)//catch the ex 
		{
            handle.errorText = ex.message;
        }
        return false;
    };
    //
    this.onchange = function (result) {
        handle.cometStatus = "data";
    };
    //
    this.close = function () {
        if (eventObject) {
           try {
               eventObject.open("GET", TUI.env.mobile.url + "/WebMobile/cometevent.srv?CmdType=Close&TunnelID=" + tunnelID + "&CometID=0&ReqType=all&ReqStr=all", false);
                //
                eventObject.setRequestHeader("If-Modified-Since", "0");
                eventObject.send(null);
                //
                var cometStatusCode = parseInt((eventObject.responseXML.documentElement.attributes["StatusCode"]==undefined)?eventObject.responseXML.documentElement.attributes[0].value:eventObject.responseXML.documentElement.attributes["StatusCode"].value);
                if (cometStatusCode == 7) {
                    handle.cometStatus = "close";
                    handle.taskCount = 0;
                    cometTask = [];
                }
                else {
                    handle.errorText = decodeURIComponent(eventObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' ');
                }
            }
            catch (ex) {
                handle.errorText = ex.message;
            }
        }
    };
    //
    this.addsrv = function (reqType, reqStr, userFunction) {
        if (eventObject) {
            try {
                //
                cometID++;
                eventObject.open("GET", TUI.env.mobile.url + "/WebMobile/cometevent.srv?CmdType=Join&TunnelID=" + tunnelID + "&CometID=" + cometID + "&ReqType=" + reqType + "&ReqStr=" + encodeURIComponent(reqStr), false);
                eventObject.setRequestHeader("If-Modified-Since", "0");
                eventObject.send(null);
                //
                var cometStatusCode = parseInt((eventObject.responseXML.documentElement.attributes["StatusCode"]==undefined)?eventObject.responseXML.documentElement.attributes[0].value:eventObject.responseXML.documentElement.attributes["StatusCode"].value);
                if (cometStatusCode == 7) {
                    handle.cometStatus = "join";
                    handle.taskCount++;
                    cometTask.push({ reqType: reqType, reqStr: reqStr, fn: userFunction, cometID: parseInt((eventObject.responseXML.documentElement.attributes["CometID"]==undefined)?eventObject.responseXML.documentElement.attributes[3].value:eventObject.responseXML.documentElement.attributes["CometID"].value) }); 
                    return true;
                }
                else {
                    handle.errorText = decodeURIComponent(eventObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' ');
                }
            }
            catch (ex) {
                handle.errorText = ex.message;
            }
        }
        //
        return false;
    };
    //
    this.removesrv = function (reqType, reqStr) {
        if (eventObject) {
            try {
                for (i = 0; i < cometTask.length; i++) {
                    if (cometTask[i].reqType = reqType
					    && cometTask[i].reqStr == reqStr) {
                        //
                        eventObject.open("GET", TUI.env.mobile.url + "/WebMobile/cometevent.srv?CmdType=Leave&TunnelID=" + tunnelID + "&CometID=" + cometTask[i].cometID + "&ReqType=" + reqType + "&ReqStr=" + encodeURIComponent(reqStr), false);
                        eventObject.setRequestHeader("If-Modified-Since", "0");
                        eventObject.send(null);
                        //
                        var cometStatusCode = parseInt((eventObject.responseXML.documentElement.attributes["StatusCode"]==undefined)?eventObject.responseXML.documentElement.attributes[0].value:eventObject.responseXML.documentElement.attributes["StatusCode"].value);
                        if (cometStatusCode == 7) {
                            handle.cometStatus = "leave";
                            handle.taskCount--;
                            cometTask.splice(i, 1);
                            return true;
                        }
                        else {
                            handle.errorText = decodeURIComponent(eventObject.responseXML.getElementsByTagName("CometData")[0].firstChild.nodeValue).replace(/\+/g, ' ');
                        }
                    }
                }
            }
            catch (ex) {
                handle.errorText = ex.message;
            }
        }
        //
        return false;
    };
};

/*************************************************************************
*
*			APP处理的方法
*
**************************************************************************/
TUI.APP = new function () {
    var myAppID = getCookie("AppID");
    this.AppID = myAppID;

    this.Close = function () {
        if (TUI.env.os[TUI.env.desktop-1].ModalWindwos != null) {
            TUI.env.os[TUI.env.desktop-1].ModalWindwos.close();
        }

        for (i = 0; i < TUI.env.os[TUI.env.desktop-1].WindowHandle.length; i++) {
            if (TUI.env.os[TUI.env.desktop-1].WindowHandle[i][1] == myAppID) {
                TUI.env.os[TUI.env.desktop-1].WindowHandle[i][0].close();
                break;
            }
        }
    };

    this.Min = function () {
        if (TUI.env.os[TUI.env.desktop-1].ModalWindwos != null) {
            TUI.env.os[TUI.env.desktop-1].ModalWindwos.hide();
        }

        for (i = 0; i < TUI.env.os[TUI.env.desktop-1].WindowHandle.length; i++) {
            if (TUI.env.os[TUI.env.desktop-1].WindowHandle[i][1] == myAppID) {
                TUI.env.os[TUI.env.desktop-1].WindowHandle[i][0].hide();
                break;
            }
        }
    };

    this.Max = function () {
        if (TUI.env.os[TUI.env.desktop-1].ModalWindwos != null) {
            TUI.env.os[TUI.env.desktop-1].ModalWindwos.max();
        }

        for (i = 0; i < TUI.env.os[TUI.env.desktop-1].WindowHandle.length; i++) {
            if (TUI.env.os[TUI.env.desktop-1].WindowHandle[i][1] == myAppID) {
                TUI.env.os[TUI.env.desktop-1].WindowHandle[i][0].max();
                break;
            }
        }
    };

    this.Normal = function () {
        if (TUI.env.os[TUI.env.desktop-1].ModalWindwos != null) {
            TUI.env.os[TUI.env.desktop-1].ModalWindwos.normal();
        }

        for (i = 0; i < TUI.env.os[TUI.env.desktop-1].WindowHandle.length; i++) {
            if (TUI.env.os[TUI.env.desktop-1].WindowHandle[i][1] == myAppID) {
                TUI.env.os[TUI.env.desktop-1].WindowHandle[i][0].normal();
                break;
            }
        }
    };

    this.FullScreen = function () {
        if (TUI.env.os[TUI.env.desktop-1].ModalWindwos != null) {
            TUI.env.os[TUI.env.desktop-1].ModalWindwos.fullscreen();
        }

        for (i = 0; i < TUI.env.os[TUI.env.desktop-1].WindowHandle.length; i++) {
            if (TUI.env.os[TUI.env.desktop-1].WindowHandle[i][1] == myAppID) {
                TUI.env.os[TUI.env.desktop-1].WindowHandle[i][0].fullscreen();
                break;
            }
        }
    };

    function getCookie(name) {
        var arr = document.cookie.match(new RegExp("(^| )" + name + "=([^;]*)(;|$)"));
        if (arr != null) return unescape(arr[2]); return null;
    }
};


jQuery.extend({
    createUploadIframe: function(id, uri)
	{
			//create frame
            var frameId = 'jUploadFrame' + id;
            /*if(window.ActiveXObject) {
				var io = document.createElement('<iframe id="' + frameId + '" name="' + frameId + '" />');
				if(typeof uri== 'boolean'){
                    io.src = 'javascript:false';
                }
                else if(typeof uri== 'string'){
                    io.src = uri;
                }
            }
            else*/ 
			{
                var io = document.createElement('iframe');
                io.id = frameId;
                io.name = frameId;
            }
            io.style.position = 'absolute';
            io.style.top = '-1000px';
            io.style.left = '-1000px';

            document.body.appendChild(io);

            return io			
    },
    createUploadForm: function(id, fileElementId)
	{
		//create form	
		var formId = 'jUploadForm' + id;
		var fileId = 'jUploadFile' + id;
		var form = $('<form  action="" method="POST" name="' + formId + '" id="' + formId + '" enctype="multipart/form-data"></form>');	
		var oldElement = $('#' + fileElementId);
		var newElement = $(oldElement).clone();
		$(oldElement).prop('id', fileId);
		$(oldElement).before(newElement);
		$(oldElement).appendTo(form);
		//set attributes
		$(form).css('position', 'absolute');
		$(form).css('top', '-1200px');
		$(form).css('left', '-1200px');
		$(form).appendTo('body');		
		return form;
    },

    ajaxFileUpload: function(s) {
        // TODO introduce global settings, allowing the client to modify them for all requests, not only timeout		
        s = jQuery.extend({}, jQuery.ajaxSettings, s);
        var id = new Date().getTime()        
		var form = jQuery.createUploadForm(id, s.fileElementId);
		var io = jQuery.createUploadIframe(id, s.secureuri);
		var frameId = 'jUploadFrame' + id;
		var formId = 'jUploadForm' + id;		
        // Watch for a new set of requests
        if ( s.global && ! jQuery.active++ )
		{
			jQuery.event.trigger( "ajaxStart" );
		}            
        var requestDone = false;
        // Create the request object
        var xml = {}   
        if ( s.global )
            jQuery.event.trigger("ajaxSend", [xml, s]);
        // Wait for a response to come back
        var uploadCallback = function(isTimeout)
		{			
			var io = document.getElementById(frameId);
            try 
			{				
				if(io.contentWindow)
				{
					 xml.responseText = io.contentWindow.document.body?io.contentWindow.document.body.innerHTML:null;
                	 xml.responseXML = io.contentWindow.document.XMLDocument?io.contentWindow.document.XMLDocument:io.contentWindow.document;
					 
				}else if(io.contentDocument)
				{
					 xml.responseText = io.contentDocument.document.body?io.contentDocument.document.body.innerHTML:null;
                	xml.responseXML = io.contentDocument.document.XMLDocument?io.contentDocument.document.XMLDocument:io.contentDocument.document;
				}						
            }catch(e)
			{
				jQuery.handleError(s, xml, null, e);
			}
            if ( xml || isTimeout == "timeout") 
			{				
                requestDone = true;
                var status;
                try {
                    status = isTimeout != "timeout" ? "success" : "error";
                    // Make sure that the request was successful or notmodified
                    if ( status != "error" )
					{
                        // process the data (runs the xml through httpData regardless of callback)
                        var data = jQuery.uploadHttpData( xml, s.dataType );    
                        // If a local callback was specified, fire it and pass it the data
                        if ( s.success )
                            s.success( data, status );
    
                        // Fire the global callback
                        if( s.global )
                            jQuery.event.trigger( "ajaxSuccess", [xml, s] );
                    } else
                        jQuery.handleError(s, xml, status);
                } catch(e) 
				{
                    status = "error";
                    jQuery.handleError(s, xml, status, e);
                }

                // The request was completed
                if( s.global )
                    jQuery.event.trigger( "ajaxComplete", [xml, s] );

                // Handle the global AJAX counter
                if ( s.global && ! --jQuery.active )
                    jQuery.event.trigger( "ajaxStop" );

                // Process result
                if ( s.complete )
                    s.complete(xml, status);

                jQuery(io).unbind()

                setTimeout(function()
									{	try 
										{
											$(io).remove();
											$(form).remove();	
											
										} catch(e) 
										{
											jQuery.handleError(s, xml, null, e);
										}									

									}, 100)

                xml = null

            }
        }
        // Timeout checker
        if ( s.timeout > 0 ) 
		{
            setTimeout(function(){
                // Check to see if the request is still happening
                if( !requestDone ) uploadCallback( "timeout" );
            }, s.timeout);
        }
        try 
		{
           // var io = $('#' + frameId);
			var form = $('#' + formId);
			$(form).prop('action', s.url);
			$(form).prop('method', 'POST');
			$(form).prop('target', frameId);
            if(form.encoding)
			{
                form.encoding = 'multipart/form-data';				
            }
            else
			{				
                form.enctype = 'multipart/form-data';
            }			
            $(form).submit();

        } catch(e) 
		{			
            jQuery.handleError(s, xml, null, e);
        }
        if(window.attachEvent){
            document.getElementById(frameId).attachEvent('onload', uploadCallback);
        }
        else{
            document.getElementById(frameId).addEventListener('load', uploadCallback, false);
        } 		
        return {abort: function () {}};	

    },

    uploadHttpData: function( r, type ) {
        var data = !type;
        data = type == "xml" || data ? r.responseXML : r.responseText;
        // If the type is "script", eval it in global context
        if ( type == "script" )
            jQuery.globalEval( data );
        // Get the JavaScript object, if JSON is used.
        if ( type == "json" )
            eval( "data = " + data );
        // evaluate scripts within html
        if ( type == "html" )
            jQuery("<div>").html(data).evalScripts();
			//alert($('param', data).each(function(){alert($(this).prop('value'));}));
        return data;
    }
});



jQuery.download = function (url, data, method) {
    //url and data options required
    if (url && data) {
        //data can be string of parameters or array/object
        data = typeof data == 'string' ? data : jQuery.param(data);
        //split params into form inputs
        var inputs = '';
        jQuery.each(data.split('&'), function () {
            var pair = this.split('=');
            inputs += '<input type="hidden" name="' + pair[0] + '" value="' + pair[1] + '" />';
        });
        //send request
        jQuery('<form action="' + url + '" method="' + (method || 'post') + '">' + inputs + '</form>')
		.appendTo('body').submit().remove();
    };
};
